﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  07/07/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Windows.Forms
{
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Runtime.InteropServices.Enums;
    using System.Runtime.InteropServices.Structures;
    using System.Runtime.Serialization.Formatters.Binary;
    using System.Security.Permissions;

    /// <summary>
    /// Represents a target for other applications to send data with the WM_COPYDATA message.
    /// </summary>
    public sealed class NativeReceiveDataSink : NativeWindow, IDisposable
    {
        /// <summary>
        /// Event raised when data is received.
        /// </summary>
        public event EventHandler<DataReceivedEventArgs> DataReceived;

        /// <summary>
        /// Initializes a new instance of the ReceiveDataNativeControl.
        /// </summary>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
        public NativeReceiveDataSink()
        {
            CreateHandle(new CreateParams());
        }

        /// <summary>
        /// This method is called when a window message is sent to the handle of the window.
        /// </summary>
        /// <param name="m">The Windows <see cref="System.Windows.Forms.Message"/> to process.</param>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
        protected override void WndProc(ref Message m)
        {
            if (m.Msg == (int)WindowsMessages.WM_COPYDATA)
            {
                //marshals the unmanaged memory object to a managed structure:
                COPYDATASTRUCT st = (COPYDATASTRUCT)Marshal.PtrToStructure(m.LParam, typeof(COPYDATASTRUCT));

                //initialize the byte array:
                byte[] dataReceived = new byte[st.cbData];

                //marshals the bytes received to a managed array:
                Marshal.Copy(st.lpData, dataReceived, 0, st.cbData);

                base.WndProc(ref m); //enables waiting application to continue before processing message

                if (DataReceived != null)
                {
                    //initialize the serialization formatter:
                    BinaryFormatter formatter = new BinaryFormatter();

                    //create a memory stream to deserialize the data
                    using (MemoryStream ms = new MemoryStream())
                    {
                        //write the data received:
                        ms.Write(dataReceived, 0, st.cbData);

                        //reset stream position:
                        ms.Position = 0;

                        //deserialize data, raise event:
                        DataReceived(this, new DataReceivedEventArgs(formatter.Deserialize(ms)));
                    }
                }

                return;
            }

            base.WndProc(ref m);
        }

        /// <summary>
        /// Disposes the native control.
        /// </summary>
        ~NativeReceiveDataSink()
        {
            Disposing(false);
        }

        /// <summary>
        /// Disposes the native control.
        /// </summary>
        public void Dispose()
        {
            Disposing(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes unmanaged and optionally, managed resources.
        /// </summary>
        /// <param name="disposing">true to release both managed and unmanaged resources; false to release only unmanaged resources.</param>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
        private void Disposing(bool disposing)
        {
            if (disposing)
                if (Handle != IntPtr.Zero)
                    DestroyHandle();
        }
    }
}
